import requests
from bs4 import BeautifulSoup
from models.company import Company
import zipfile
import os
import shutil


URL = "https://www.kap.org.tr/tr/api/member/filter/"
DOWNLOAD_URL = "https://www.kap.org.tr/tr/api/home-financial/download-file/"


def _download_xls(mkkMemberOid: str, year: str) -> str:
    # Burada dosyayı indiriyor
    data = requests.get(f"{DOWNLOAD_URL}{mkkMemberOid}/{year}/T", stream=False)
    data.raise_for_status()
    return data.content


def _search_oid(company: Company) -> dict:
    response = requests.get(f"{URL}{company.code}")
    response.raise_for_status()
    oid = response.json()[0]["mkkMemberOid"]
    if not oid:
        raise Exception(f"Company {company.code} not found.")
    return oid


def _extract_data(data: str) -> dict:
    soup = BeautifulSoup(data, 'html.parser')
    table = soup.find(
        'table', {'class': 'financial-table tbl_general_role_210015'})
    extracted_data = []
    rows = table.find_all('tr')[7:]
    for row in rows:
        cells = row.find_all('td')
        if cells:
            key_cell = cells[1].find(
                'div', {'class': 'gwt-Label'}) if len(cells) > 1 else None
            value_cell = cells[7] if len(cells) > 7 else None

            key = key_cell.text.strip() if key_cell else ""
            value = value_cell.text.strip() if value_cell else ""

            if key or value:
                extracted_data.append({"key": key, "value": value})

    return extracted_data


async def get_financial_report(company: Company, year: str = "2023") -> dict:
    oid = _search_oid(company)
    content = _download_xls(oid, year=year)
    zip_file_path = f"{company.code}_financial_report.zip"
    with open(zip_file_path, "wb") as file:
        file.write(content)

    with zipfile.ZipFile(zip_file_path, 'r') as zip_ref:
        zip_ref.extractall(f"{company.code}_financial_report")
        extracted_data = {}
        for file_name in zip_ref.namelist():
            if file_name.endswith('.xls'):
                with zip_ref.open(file_name) as file:
                    period = f"period_{file_name.split('_')[-1]}"
                    period = period.replace('.xls', '')
                    data = file.read()
                    extracted_data[period] = _extract_data(data)

    os.remove(zip_file_path)
    shutil.rmtree(f"{company.code}_financial_report")
    return extracted_data



import asyncio

from kap_client import KapClient


async def sample_get_company():
    client = KapClient()
    company = await client.get_company("BIMAS")
    message = f"Sample get company: {company}"
    print(message)

async def sample_get_company_info():
    client = KapClient()
    company = await client.get_company("BIMAS")
    info = await client.get_company_info(company)
    message = f"Sample get company info: {info}"
    print(message)

async def sample_get_financial_report():
    client = KapClient()
    company = await client.get_company("BIMAS")
    report = await client.get_financial_report(company, "2022")
    message = f"Sample get financial report: {report}"
    print(message)

async def sample_get_indices():
    client = KapClient()
    indices = await client.get_indices()
    message = f"Sample get indices: {indices}"
    print(message)

async def main():
    await sample_get_company()
    await sample_get_company_info()
    await sample_get_indices()
    await sample_get_financial_report()



if __name__ == "__main__":
    asyncio.run(main())






import logging
from models.indices import Indice
from pyppeteer import launch
from bs4 import BeautifulSoup
from models.company import Company
from typing import Dict


URL = "https://www.kap.org.tr/tr/Endeksler"


async def scrape_indices() -> list[Indice]:
    indices = []
    browser = None

    try:
        browser = await launch()
        page = await browser.newPage()
        await page.goto(URL, {"waitUntil": "domcontentloaded"})
        await page.waitForSelector('#indicesTable', timeout=10000)
        await page.evaluate("document.querySelector('#stickyDropdown > div > div').click()")
        content = await page.content()
        soup = BeautifulSoup(content, 'html.parser')
        table = soup.find(id='indicesTable')
        indices: Dict[str, Indice] = {}
        last_index: str = ""
        for row in table.find_all('tr')[2:]:
            cols = row.find_all('td')
            if len(cols) == 1:
                spans = cols[0].find_all('span')
                last_index = spans[0].text.strip() if spans else ""
                if last_index and last_index not in indices:
                    indices[last_index] = Indice(last_index)
                    indices[last_index].companies = []
            elif len(cols) == 4:
                if not last_index:
                    logging.error("Last index not set, skipping row")
                    continue
                code = cols[1].text.strip()
                indices[last_index].companies.append(code)

        for value in soup.select('div[class*="select__option"]')[1:]:
            span = value.find_all('span')
            if len(span) < 2:
                continue
            index = span[0].text.strip() if span else ""
            if index not in indices:
                continue
            indices[index].code = span[1].text.strip() if span else ""

    except Exception as e:
        logging.error(f"Scraping failed: {e}")
    finally:
        if browser:
            await browser.close()

    return list(indices.values())



from asyncio.log import logger
from models.company import Company
from pyppeteer import launch
from models.company import Company
from bs4 import BeautifulSoup
from models.company_info import CompanyInfo

_GENERAL_URL = "https://www.kap.org.tr/tr/sirket-bilgileri/ozet/"

def _find_cells(
    soup: BeautifulSoup,
    string: str,
) -> list[str]:
    data = soup.find('h3', string=string)
    if data:
        next_container = data.find_next(["p", "div"])
        text_elements = next_container.find_all(string=True, recursive=True)
        return [a.text for a in text_elements]
    return [""]
""

async def scrape_company_info(company: Company) -> CompanyInfo:
    browser = None
    try:
        browser = await launch()
        page = await browser.newPage()
        await page.goto(_GENERAL_URL + company.path, {"waitUntil": "domcontentloaded"})
        await page.waitForSelector('#financialTable', timeout=10000)
        content = await page.content()
        soup = BeautifulSoup(content, 'html.parser')
        info = CompanyInfo()


        info.address = _find_cells(soup, 'Merkez Adresi')[0]
        info.mail = _find_cells(soup, 'Elektronik Posta Adresi')
        info.website = _find_cells(soup, 'İnternet Adresi')[0]
        info.companys_duration = _find_cells(soup, 'Şirketin Süresi')[0]
        info.independent_audit_firm = _find_cells(soup, 'Bağımsız Denetim Kuruluşu')[0]
        info.indices = _find_cells(soup, 'Şirketin Dahil Olduğu Endeksler')
        info.sectors = _find_cells(soup, 'Şirketin Sektörü')
        info.equity_market = _find_cells(soup, 'Sermaye Piyasası Aracının İşlem Gördüğü Pazar')[0]
        return info
    except Exception as e:
        logger.error(f"Error scraping company info: {e}")
        return None
    finally:
        if browser:
            await browser.close()






import diskcache
from _companies import scrape_companies
from _indices import scrape_indices
from _company_info import scrape_company_info
from _financial_report import get_financial_report
from models.company import Company
from models.indices import Indice
from models.company_info import CompanyInfo
from typing import Optional

_CACHE_KEY = "kap_cache"

class KapClient:

    def __init__(
        self,
        cache_expiry=3600
    ):
        self.cache = diskcache.Cache(_CACHE_KEY)
        self.cache_expiry = cache_expiry

    async def get_companies(self) -> list[Company]:
        key = "companies"
        cached_companies = self.cache.get(key=key)
        if cached_companies:
            return cached_companies
        companies = await scrape_companies()
        self.cache.set(key, companies, expire=self.cache_expiry)
        return companies

    async def get_indices(self) -> list[Indice]:
        key = "indices"
        cached_indices = self.cache.get(key=key)
        if cached_indices:
            return cached_indices
        indices = await scrape_indices()
        self.cache.set(key, indices, expire=self.cache_expiry)
        return indices

    async def get_company(self, code: str) -> Optional[Company]:
        companies = await self.get_companies()
        for company in companies:
            if company.code == code:
                return company
        return None

    async def get_indice(self, code: str) -> Optional[Indice]:
        indices = await self.get_indices()
        for indice in indices:
            if indice.code == code:
                return indice
        return None

    async def get_company_info(self, company: Company) -> Optional[CompanyInfo]:
        key = f"infos_{company.code}"
        cached_company_info = self.cache.get(key=key)
        if cached_company_info:
            return cached_company_info
        company_info = await scrape_company_info(company)
        self.cache.set(key, company_info, expire=self.cache_expiry)
        return company_info

    async def get_financial_report(self, company: Company, year: str = "2023") -> dict:
        key = f"financial_report_{company.code}_{year}"
        cached_financial_report = self.cache.get(key=key)
        if cached_financial_report:
            return cached_financial_report
        financial_report = await get_financial_report(company=company, year=year)
        self.cache.set(key, financial_report, expire=self.cache_expiry)
        return financial_report


    def clear_cache(self):
        self.cache.clear()





from setuptools import setup, find_packages

setup(
    name='kap_sdk',
    version='0.1.2',
    description='Kap Data Scrapping',
    packages=find_packages(),
    install_requires=[
        'requests',
        'beautifulsoup4',
        'pyppeteer',
        "diskcache"
    ],
)



import logging
from pyppeteer import launch
from bs4 import BeautifulSoup
from models.company import Company

URL = "https://www.kap.org.tr/tr/bist-sirketler"

def _parse_row(row):
    cols = row.find_all('td')
    if len(cols) < 4:
        return None
    return Company(
        code=cols[0].text.strip(),
        name=cols[1].text.strip(),
        city=cols[2].text.strip(),
        independent_audit_firm=cols[3].text.strip(),
        path=cols[1].find('a')['href'].strip().split('/')[-1]
    )


async def scrape_companies() -> list[Company]:
    companies = []
    browser = None

    try:
        browser = await launch()
        page = await browser.newPage()

        await page.goto(URL, {"waitUntil": "domcontentloaded"})
        await page.waitForSelector('#financialTable', timeout=10000)
        content = await page.content()
        soup = BeautifulSoup(content, 'html.parser')
        table = soup.find(id='financialTable')

        for row in table.find_all('tr')[1:]:
            company = _parse_row(row)
            if company:
                companies.append(company)
    except Exception as e:
        logging.error(f"Scraping failed: {e}")
    finally:
        if browser:
            await browser.close()

    return companies
